from src.settings_validator import settings_validator


class settings:
    # Полное имя
    __first_name = ""
    # ИНН
    __tax_id = 0
    # БИК
    __bank_id = 0
    # Номер счета
    __bank_account_id = 0
    # Номер корресп. счета
    __corresp_account_id = 0
    # Имя собственности
    __property_name = ""
    # Тип собственности
    __property_type = ""


    # Валидатор
    __vtor = None

    def __init__(self):
        self.__vtor = settings_validator()


    def __validate(value, exp_type=None, length=None):
        """
            Валидация аргумента по типу и/или длине
        Args:
            value (any): Передаваемый аргумент
            type (object): Ожидаемый тип
            length (int): Ожидаемая длина
        Raises:
            TypeError: Некорректный тип аргумента
            ValueError: Некорректная длина аргумента
        Returns:
            True или Exception взависимости от результата валидации
        """

        # Проверяем соответствие типа
        if exp_type is not None and not isinstance(value, exp_type):
            raise TypeError("Некорректный тип аргумента")
        
        # Проверяем соответствие длины
        if length is not None and len(str(value)) != length:
            raise ValueError("Некорректная длина аргумента")
        
        return True
    
    @property
    def first_name(self):
        return self.__first_name
    
    @first_name.setter
    def first_name(self, value: str):
        """
            Полное наименование
        Args:
            value (str): Полное наименование

        Raises:
            TypeError: Некорректный аргумент
        """

        self.__vtor.check_type(value, str)
        
        self.__first_name = value.strip()

    @property
    def tax_id(self):
        return self.__tax_id
    
    @tax_id.setter
    def tax_id(self, value: int):
        """
            ИНН
        Args:
            value (int): Значение ИНН, 12 символов

        Raises:
            TypeError: Некорректный тип аргумента
            ValueError: Некорректная длина аргумента
        """

        self.__vtor.check_type(value, int)
        self.__vtor.check_length(value, 12)

        self.__tax_id = value
    
    @property
    def bank_id(self):
        return self.__bank_id
    
    @bank_id.setter
    def bank_id(self, value: int):
        """
            БИК
        Args:
            value (int): Значение БИК, 9 символов

        Raises:
            TypeError: Некорректный тип аргумента
            ValueError: Некорректная длина аргумента
        """

        settings.__validate(value, int, 9)
        self.__vtor.check_type(value, int)
        self.__vtor.check_length(value, 9)

        self.__bank_id = value

    
    @property
    def bank_account_id(self):
        return self.__bank_account_id
    
    @bank_account_id.setter
    def bank_account_id(self, value: int):
        """
            Номер банковского счета
        Args:
            value (int): Номер банковского счета, 11 символов

        Raises:
            TypeError: Некорректный тип аргумента
            ValueError: Некорректная длина аргумента
        """

        self.__vtor.check_type(value, int)
        self.__vtor.check_length(value, 11)

        self.__bank_account_id = value
        

    @property
    def corresp_account_id(self):
        return self.__corresp_account_id
    
    @corresp_account_id.setter
    def corresp_account_id(self, value: int):
        """
            Номер корреспондентского счёта
        Args:
            value (int): Номер корреспондентского счёта, 11 символов

        Raises:
            TypeError: Некорректный тип аргумента
            ValueError: Некорректная длина аргумента
        """

        self.__vtor.check_type(value, int)
        self.__vtor.check_length(value, 11)

        self.__corresp_account_id = value


    @property
    def property_name(self):
        return self.__property_name
    
    @property_name.setter
    def property_name(self, value: str):
        """
            Наименование собственности
        Args:
            value (str): Наименование собственности

        Raises:
            TypeError: Некорректный тип аргумента
        """

        self.__vtor.check_type(value, str)

        self.__property_name = value

    
    @property
    def property_type(self):
        return self.__property_type
    
    @property_type.setter
    def property_type(self, value: str):
        """
            Тип собственности
        Args:
            value (str): Тип собственности, 5 символов

        Raises:
            TypeError: Некорректный тип аргумента
            ValueError: Некорректная длина аргумента
        """

        self.__vtor.check_type(value, str)
        self.__vtor.check_length(value, 5)

        self.__property_type = value
